tree grammar EngineConfigPointSchemaWalker ;
options {
	tokenVocab = EngineConfigPointSchema ;
	ASTLabelType = CommonTree ;
}

@header {
package com.springsource.engine.config.core.validate;

import com.springsource.engine.common.math.Range;
import com.springsource.engine.common.math.Version;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
}

@members {
	private String configPointName;
	private ConfigValidator configPointValidator;
	
	// Stacks are used to hold the last of various kinds of nodes but should never be more than one element deep.
	Stack<ConfigValidator> configurationSchemaStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> mapStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> listStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> valueStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> stringStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> anyStringStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> enumerationStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> integerStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> integerRangeStack = new Stack<ConfigValidator>();
	Stack<ConfigValidator> versionStack = new Stack<ConfigValidator>();
	Stack<Version> versionValueStack = new Stack<Version>();
	Stack<ConfigValidator> versionRangeStack = new Stack<ConfigValidator>();
	
	public String getConfigPointName() {
		return this.configPointName;
	}
	
	public ConfigValidator getConfigPointValidator() {
		return this.configPointValidator;
	}
}

configPointSchema
scope {
	String configPointName;
	ConfigValidator configPointValidator;
}
@after {
	this.configPointName = $configPointSchema::configPointName;
	this.configPointValidator = $configPointSchema::configPointValidator;
}
	:	^( ConfigPointSchema 	IDENTIFIER { $configPointSchema::configPointName = $IDENTIFIER.text; }
					configurationSchema { $configPointSchema::configPointValidator = configurationSchemaStack.pop(); }
		 )
	;
	
configurationSchema 
	:	mapSchema { configurationSchemaStack.push(mapStack.pop()); } | listSchema { configurationSchemaStack.push(listStack.pop()); }
	;
	
mapSchema
scope {
	Map<String, ConfigValidator> validatorMap;
}
@init {
	$mapSchema::validatorMap = new HashMap<String, ConfigValidator>();
}
@after {
	mapStack.push(new StandardMapConfigNodeValidator($mapSchema::validatorMap));
}
	:	^( MAPSCHEMA mapletSchema+ )
	;

mapletSchema
scope {
	String key;
	ConfigValidator valueValidator;
}
@after {
	$mapSchema::validatorMap.put($mapletSchema::key,$mapletSchema::valueValidator);
}
	:	^( MAPLETSCHEMA key valueSchema { $mapletSchema::valueValidator = valueStack.pop(); } )
	;

valueSchema
	:	stringSchema { valueStack.push(stringStack.pop()); } | configurationSchema {valueStack.push(configurationSchemaStack.pop());}
	;

listSchema
@after {
	listStack.push(new StandardListConfigNodeValidator(valueStack.pop()));
}
	: 	^( LISTSCHEMA valueSchema )
	;
	
key	: 	^( KEY IDENTIFIER )
		{ $mapletSchema::key = $IDENTIFIER.text; }
	;
			
stringSchema  :	^( STRINGSCHEMA 	( anyString { stringStack.push(anyStringStack.pop()); } ) ?
									( enumeration { stringStack.push(enumerationStack.pop()); } ) ? 
									( integer { stringStack.push(integerStack.pop()); } ) ?
									( integerRange { stringStack.push(integerRangeStack.pop()); } ) ?
									( version { stringStack.push(versionStack.pop()); } ) ?
									( versionRange { stringStack.push(versionRangeStack.pop()); } ) ?
				 )
	;
	
anyString : ANYSTRING
			{ anyStringStack.push(new StandardScalarConfigNodeValidator()); }
	;

enumeration
scope {
	Set<String> values;
}
@init {
	$enumeration::values = new HashSet<String>();
}
@after {
	enumerationStack.push(new StandardEnumerationConfigNodeValidator($enumeration::values));
}	
	: ^( ENUM ( IDENTIFIER { $enumeration::values.add($IDENTIFIER.text); } ) + )
	;

integer : INTEGER { integerStack.push(new StandardIntegerConfigNodeValidator()); }
	;
	
integerRange
scope {
	int floor;
	boolean floorInc;
		
	int ceiling;
	boolean ceilingInc;
}
@after {
	this.integerRangeStack.push(new StandardIntegerRangeConfigNodeValidator(new Range<Integer>($integerRange::floor, $integerRange::floorInc, $integerRange::ceiling, $integerRange::ceilingInc)));
}	
	: ^( INTEGERRANGE integerRangeStart integerRangeEnd)
	;
	
integerRangeStart : 	^( INTGE NUMBER { $integerRange::floor = Integer.parseInt($NUMBER.text); $integerRange::floorInc = true; } (POSITIVE | NEGATIVE { $integerRange::floor = -$integerRange::floor; }) ) |
			^( INTG NUMBER { $integerRange::floor = Integer.parseInt($NUMBER.text); $integerRange::floorInc = false; } (POSITIVE | NEGATIVE { $integerRange::floor = -$integerRange::floor; }) )
	;

integerRangeEnd : 	^( INTLE NUMBER { $integerRange::ceiling = Integer.parseInt($NUMBER.text); $integerRange::ceilingInc = true; } (POSITIVE | NEGATIVE { $integerRange::ceiling = -$integerRange::ceiling; }) ) |
			^( INTL NUMBER { $integerRange::ceiling = Integer.parseInt($NUMBER.text); $integerRange::ceilingInc = false; } (POSITIVE | NEGATIVE { $integerRange::ceiling = -$integerRange::ceiling; }) )
	;
	
version : VERSION { versionStack.push(new StandardVersionConfigNodeValidator()); }
	;
	
versionValue
scope {
	List<Integer> components;
}
@init {
	$versionValue::components = new ArrayList<Integer>();
}
@after {
	this.versionValueStack.push(new Version($versionValue::components));
}
	: ^( VERSIONVALUE ( NUMBER { $versionValue::components.add(Integer.parseInt($NUMBER.text)); } )+ )
	; 

versionRange	
scope {
	Version floor;
	boolean floorInc;
		
	Version ceiling;
	boolean ceilingInc;
}
@after {
	this.versionRangeStack.push(new StandardVersionRangeConfigNodeValidator(new Range<Version>($versionRange::floor, $versionRange::floorInc, $versionRange::ceiling, $versionRange::ceilingInc)));
}
	: ^( VERSIONRANGE versionRangeStart versionRangeEnd)
	;
	
versionRangeStart : 	^( VERGE versionValue { $versionRange::floor = this.versionValueStack.pop(); $versionRange::floorInc = true; } ) |
			^( VERG versionValue { $versionRange::floor = this.versionValueStack.pop(); $versionRange::floorInc = false; } )
	;
	
versionRangeEnd : 	^( VERLE versionValue { $versionRange::ceiling = this.versionValueStack.pop(); $versionRange::ceilingInc = true; } ) |
			^( VERL versionValue { $versionRange::ceiling = this.versionValueStack.pop(); $versionRange::ceilingInc = false; } )
	;
